<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


/**
 * This script takes some POST variables and validates the commit data BEFORE committing.
 * It is used to interact with external SCM scripts.
 */
 
require_once('plugins/scm/lib/SCMRepositoryFactory.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmPreference.php');

class PreScmCommit {
	private $project;
	
	function __construct($project) {
		$this->project = $project;
		
		$repos = ScmRepositoryFactory::getRepository($this->project);
		if (is_null($repos)) {
			$this->returnError('Unable to retrieve repository object for project '.$this->project->getUnixName());
		}
		
		$scm_type = $repos->getType();


		if (!isset($_POST['username'])) {
			$this->returnError('User name not set in POST data');
		}
		
		$username = $_POST['username'];
		$user = UserPeer::retrieveByUnixName($username);

		// Only check for the serialized files if it's not CVS (in CVS a separate check is made for
		// each directory we're committing to)
		if (strtolower($scm_type) != 'cvs') {
			if (!isset($_POST['serialized_files'])) {
				$this->returnError('File information not set in POST data');
			} elseif (!isset($_POST['log'])) {
				$this->returnError('Log information not set in POST data');
			}
			$log = $_POST['log'];
			$files = unserialize($_POST['serialized_files']);
		} else {
			// We have two actions for CVS: ACL check and log check. Each of them has
			// its specific parameters
			if (!isset($_POST['cvsaction'])) {
				$this->returnError('CVS action not set in POST data');
			} elseif ($_POST['cvsaction'] != 'aclcheck' && $_POST['cvsaction'] != 'logcheck') {
				$this->returnError('Invalid CVS action');
			} elseif ($_POST['cvsaction'] == 'aclcheck' && !isset($_POST['path'])) {
				$this->returnError('CVS commit path not set for ACL check');
			} elseif ($_POST['cvsaction'] == 'logcheck' && !isset($_POST['log'])) {
				$this->returnError('CVS commit log not set for log check');
			}
			
			if ($_POST['cvsaction'] == 'logcheck') {
				$log = $_POST['log'];
			}
		}
		
		try {
			if ($repos->getType() != 'cvs' || $_POST['cvsaction'] == 'logcheck') {
				// See if we must check for a tracker item ID in the message log
				$preferences = ScmPreferencePeer::retrieveByPk($this->project->getProjectId());
				if (!is_null($preferences) && $preferences->getRequireTrackerItemId()) {
					$invalid_ids = array();
					if (!ScmCommit::checkTrackerItemIds($log, $project, $invalid_ids)) {
						$error = '';
						if (count($invalid_ids)>0) {
							$error .= 'The following Tracker Item Ids are not valid for this Project or you are violating commit rules:'."\n";
							$error .= '[#'.implode('], [#', $invalid_ids)."]\n";
						} else {
							$error .= 'Commit not related to any Tracker Item.'."\n";
						}
						$error .= 'Please specify one or more valid Tracker Item Ids in the commit '."\n";
						if ($repos->getType() == 'cvs') {
							$error .= 'log as "[#nnnn]". For example:'."\n";
							$error .= '$ cvs commit -m "Fixed [#nnnn]"'."\n";
						} else if ($repos->getType() == 'svn') {
							$error .= 'log as "[#nnnn]". For example:'."\n";
							$error .='$ svn commit -m "Fixed [#2708]"'."\n";
						} else if ($repos->getType() == 'perforce') {
							$changelist_num = $_POST['changelist_num'];
							$error .= 'log as "[#nnnn]". Edit the description with the command "p4 changelist '.$changelist_num.'".'."\n";
							$error .= 'and then submit the changes with the command "p4 submit -c '.$changelist_num.'".'."\n";
						}
						$this->returnError($error);
					}
				}
			}
			
			// Now check for the ACL permissions
			if (!is_null($user)) {
				// Sets the session user as the user that committed the files
				Session::getSession()->setUser($user);
				
				if ($repos->getType() == 'cvs') {
					if ($_POST['cvsaction'] == 'aclcheck') {
						// For CVS we only need to check the path
						if (!$repos->userCanCommit($_POST['path'])) {
							$error = "\n".'You don\'t have permissions to commit to '.$_POST['path']."\n";
							$this->returnError($error);
						}
					}
				} else {
					$disallowed_paths = array();
					foreach ($files as $file) {
						$dir = dirname($file['path']);
						if (!$repos->userCanCommit($dir) && !in_array($dir, $disallowed_paths)) {
							$disallowed_paths[] = $dir;
						}
					}
					
					if (count($disallowed_paths) > 0) {
						// User can't commit to some directories / modules
						$error = "\n".'You don\'t have permissions to commit to the following directories:';
						$error .= "\n* ".implode("\n* ", $disallowed_paths)."\n";
						$this->returnError($error);
					}
				}
			} else {
				//$this->returnError("User $username doesn't exist on the GForge server");
			}

		} catch (Exception $e) {
			$this->returnError($e->getMessage());
		}
		
		$this->returnSuccess();
	}
	
	private function checkLog($log) {
	}


	private function returnError($log, $error_code=1) {
		$result = array(
					'error' => $error_code,
					'log' => $log
				);
		echo serialize($result);
		die();
	}
	
	private function returnSuccess($message='') {
		$result = array(
					'error' => 0,
					'log' => $message
				);
		echo serialize($result);
		die();
	}	
}

$project = GFUrl::getUrl()->getProject();
$scmcommit = new PreScmCommit($project);



?>
